@@ -9,13 +9,14 @@ from autots.datasets import (
     load_monthly,
     load_yearly,
     load_weekly,
+    load_weekdays,
 )
 
 from autots.evaluator.auto_ts import AutoTS
 from autots.tools.transform import GeneralTransformer, RandomTransform
 from autots.tools.shaping import long_to_wide
 
-__version__ = '0.3.0'
+__version__ = '0.3.1'
 
 TransformTS = GeneralTransformer
 
@@ -25,6 +26,7 @@ __all__ = [
     'load_yearly',
     'load_hourly',
     'load_weekly',
+    'load_weekdays',
     'AutoTS',
     'TransformTS',
     'GeneralTransformer',
