@@ -4,6 +4,7 @@ import pandas as pd
 import random
 import copy
 import json
+import sys
 
 from autots.tools.shaping import (
     long_to_wide,
@@ -40,7 +41,8 @@ class AutoTS(object):
             More runs = longer runtime, generally better accuracy.
         no_negatives (bool): if True, all negative predictions are rounded up to 0.
         constraint (float): when not None, use this value * data st dev above max or below min for constraining forecast values. Applied to point forecast only, not upper/lower forecasts.
-        ensemble (str): None, 'simple', 'distance'
+        ensemble (str): None or list or comma-separated string containing:
+            'auto', 'simple', 'distance', 'horizontal-max', 'probabilistic-max', "hdist"
         initial_template (str): 'Random' - randomly generates starting template, 'General' uses template included in package, 'General+Random' - both of previous. Also can be overriden with self.import_template()
         random_seed (int): random seed allows (slightly) more consistent results.
         holiday_country (str): passed through to Holidays package for some models.
@@ -127,6 +129,8 @@ class AutoTS(object):
         self.constraint = constraint
         self.random_seed = random_seed
         self.holiday_country = holiday_country
+        if isinstance(ensemble, list):
+            ensemble = ",".join(ensemble)
         self.ensemble = str(ensemble).lower()
         self.subset = subset
         self.na_tolerance = na_tolerance
@@ -264,9 +268,6 @@ class AutoTS(object):
         ]
         self.initial_results = TemplateEvalObject()
 
-        if verbose >= 0 and ensemble is not None and "GluonTS" in self.model_list:
-            print("WARNING: GluonTS may cause errors in ensembling")
-
         if verbose > 2:
             print('"Hello. Would you like to destroy some evil today?" - Sanderson')
 
@@ -357,7 +358,7 @@ class AutoTS(object):
         if date_col is None and value_col is None:
             df_wide = pd.DataFrame(df)
             assert (
-                type(df.index) is pd.DatetimeIndex
+                type(df_wide.index) is pd.DatetimeIndex
             ), "df index is not pd.DatetimeIndex"
         else:
             df_wide = long_to_wide(
@@ -403,10 +404,8 @@ class AutoTS(object):
             }
 
         # handle categorical data if present
-        self.categorical_transformer = NumericTransformer(verbose=self.verbose).fit(
-            df_wide
-        )
-        df_wide_numeric = self.categorical_transformer.transform(df_wide)
+        self.categorical_transformer = NumericTransformer(verbose=self.verbose)
+        df_wide_numeric = self.categorical_transformer.fit_transform(df_wide)
 
         # replace any zeroes that occur prior to all non-zero values
         if self.remove_leading_zeroes:
@@ -457,9 +456,14 @@ class AutoTS(object):
             verbose=self.verbose,
         )
         try:
-            future_regressor = pd.DataFrame(future_regressor)
+            if not isinstance(future_regressor, pd.DataFrame):
+                future_regressor = pd.DataFrame(future_regressor)
             if not isinstance(future_regressor.index, pd.DatetimeIndex):
                 future_regressor.index = df_subset.index
+            # handle any non-numeric data, crudely
+            future_regressor = NumericTransformer(verbose=self.verbose).fit_transform(
+                future_regressor
+            )
             self.future_regressor_train = future_regressor
             future_regressor_train = future_regressor.reindex(index=df_train.index)
             future_regressor_test = future_regressor.reindex(index=df_test.index)
@@ -1015,6 +1019,8 @@ or otherwise increase models available."""
                     self.used_regressor_check = True
             except KeyError:
                 pass
+        # clean up any remaining print statements
+        sys.stdout.flush()
         return self
 
     def _regr_param_check(self, param_dict):
@@ -1050,7 +1056,7 @@ or otherwise increase models available."""
             prediction_interval (float): interval of upper/lower forecasts.
                 defaults to 'self' ie the interval specified in __init__()
                 if prediction_interval is a list, then returns a dict of forecast objects.
-            future_regressor (numpy.Array): additional regressor, not used
+            future_regressor (numpy.Array): additional regressor
             hierarchy: Not yet implemented
             just_point_forecast (bool): If True, return a pandas.DataFrame of just point forecasts
 
@@ -1069,7 +1075,13 @@ or otherwise increase models available."""
             future_regressor = []
             self.future_regressor_train = []
         else:
-            future_regressor = pd.DataFrame(future_regressor)
+            if not isinstance(future_regressor, pd.DataFrame):
+                future_regressor = pd.DataFrame(future_regressor)
+            # handle any non-numeric data, crudely
+            future_regressor = NumericTransformer(verbose=self.verbose).fit_transform(
+                future_regressor
+            )
+            # make sure training regressor fits training data index
             self.future_regressor_train = self.future_regressor_train.reindex(
                 index=self.df_wide_numeric.index
             )
@@ -1133,6 +1145,7 @@ or otherwise increase models available."""
             df_forecast.upper_forecast = trans.inverse_transform(
                 df_forecast.upper_forecast
             )
+            sys.stdout.flush()
             if just_point_forecast:
                 return df_forecast.forecast
             else:
@@ -1233,14 +1246,14 @@ or otherwise increase models available."""
             )
 
     def import_template(
-        self, filename: str, method: str = "Add On", enforce_model_list: bool = True
+        self, filename: str, method: str = "add_on", enforce_model_list: bool = True
     ):
         """Import a previously exported template of model parameters.
         Must be done before the AutoTS object is .fit().
 
         Args:
             filename (str): file location (or a pd.DataFrame already loaded)
-            method (str): 'Add On' or 'Only'
+            method (str): 'add_on' or 'only' - "add_on" keeps `initial_template` generated in init. "only" uses only this template.
             enforce_model_list (bool): if True, remove model types not in model_list
         """
         if isinstance(filename, pd.DataFrame):
@@ -1274,7 +1287,7 @@ or otherwise increase models available."""
                     "Len 0. Model_list does not match models in template! Try enforce_model_list=False."
                 )
 
-        if method.lower() in ['add on', 'addon']:
+        if method.lower() in ['add on', 'addon', 'add_on']:
             self.initial_template = self.initial_template.merge(
                 import_template,
                 how='outer',
@@ -1285,10 +1298,10 @@ or otherwise increase models available."""
             self.initial_template = self.initial_template.drop_duplicates(
                 subset=self.template_cols
             )
-        elif method.lower() in ['only', 'user only']:
+        elif method.lower() in ['only', 'user only', 'user_only', 'import_only']:
             self.initial_template = import_template
         else:
-            return ValueError("method must be 'add on' or 'only'")
+            return ValueError("method must be 'add_on' or 'only'")
 
         return self
 
@@ -1514,9 +1527,9 @@ class AutoTSIntervals(object):
 def fake_regressor(
     df,
     forecast_length: int = 14,
-    date_col: str = 'datetime',
-    value_col: str = 'value',
-    id_col: str = 'series_id',
+    date_col: str = None,
+    value_col: str = None,
+    id_col: str = None,
     frequency: str = 'infer',
     aggfunc: str = 'first',
     drop_most_recent: int = 0,
@@ -1529,7 +1542,9 @@ def fake_regressor(
 
     if date_col is None and value_col is None:
         df_wide = pd.DataFrame(df)
-        assert type(df.index) is pd.DatetimeIndex, "df index is not pd.DatetimeIndex"
+        assert (
+            type(df_wide.index) is pd.DatetimeIndex
+        ), "df index is not pd.DatetimeIndex"
     else:
         df_wide = long_to_wide(
             df,
