@@ -3,7 +3,7 @@ import datetime
 import numpy as np
 import pandas as pd
 import json
-from autots.evaluator.auto_model import PredictionObject
+from autots.models.base import PredictionObject
 
 
 def BestNEnsemble(
@@ -136,7 +136,7 @@ def horizontal_classifier(df_train, known: dict, method: str = "whatever"):
 
     Args:
         df_train (pandas.DataFrame): historical data about the series. Columns = series_ids.
-        known (dict): dict of series_id: classifier outcome including some but not all series.
+        known (dict): dict of series_id: classifier outcome including some but not all series in df_train.
 
     Returns:
         dict.
@@ -156,12 +156,48 @@ def horizontal_classifier(df_train, known: dict, method: str = "whatever"):
     clf.fit(Xt, Y)
     result = clf.predict(Xf)
     result_d = dict(zip(Xf.index.tolist(), result))
+    # since this only has estimates, overwrite with known that includes more
     final = {**result_d, **known}
     # temp = pd.DataFrame({'series': list(final.keys()), 'model': list(final.values())})
     # temp2 = temp.merge(X, left_on='series', right_index=True)
     return final
 
 
+def generalize_horizontal(
+    df_train, known_matches: dict, available_models: list, full_models: list = None
+):
+    """generalize a horizontal model trained on a subset of all series
+
+    Args:
+        df_train (pd.DataFrame): time series data
+        known_matches (dict): series:model dictionary for some to all series
+        available_models (dict): list of models actually available
+        full_models (dict): models that are available for every single series
+    """
+    org_idx = df_train.columns
+    org_list = org_idx.tolist()
+    # remove any unavailable models or unnecessary series
+    known_matches = {ser: mod for ser, mod in known_matches.items() if ser in org_list}
+    k = {ser: mod for ser, mod in known_matches.items() if mod in available_models}
+    # check if any series are missing from model list
+    if not k:
+        raise ValueError("Horizontal template has no models matching this data!")
+    if len(set(org_list) - set(list(k.keys()))) > 0:
+        # filter down to only models available for all
+        # print(f"Models not available: {[ser for ser, mod in known_matches.items() if mod not in available_models]}")
+        # print(f"Series not available: {[ser for ser in df_train.columns if ser not in list(known_matches.keys())]}")
+        if full_models is not None:
+            k2 = {ser: mod for ser, mod in k.items() if mod in full_models}
+        else:
+            k2 = k.copy()
+        all_series_part = horizontal_classifier(df_train, k2)
+        # since this only has "full", overwrite with known that includes more
+        all_series = {**all_series_part, **k}
+    else:
+        all_series = known_matches
+    return all_series
+
+
 def HorizontalEnsemble(
     ensemble_params,
     forecasts_list,
@@ -171,22 +207,27 @@ def HorizontalEnsemble(
     forecasts_runtime,
     prediction_interval,
     df_train=None,
+    prematched_series: dict = None,
 ):
     """Generate forecast for per_series ensembling."""
+    # this is meant to fill in any failures
     available_models = list(forecasts.keys())
-    known_matches = ensemble_params['series']
+    train_size = df_train.shape
+    # print(f"running inner generalization with training size: {train_size}")
+    full_models = [
+        mod for mod, fcs in forecasts.items() if fcs.shape[1] == train_size[1]
+    ]
+    if not full_models:
+        full_models = available_models  # hope it doesn't need to fill
+    # print(f"FULLMODEL {len(full_models)}: {full_models}")
+    if prematched_series is None:
+        prematched_series = ensemble_params['series']
+    all_series = generalize_horizontal(
+        df_train, prematched_series, available_models, full_models
+    )
+    # print(f"ALLSERIES {len(all_series.keys())}: {all_series}")
+
     org_idx = df_train.columns
-    org_list = org_idx.tolist()
-    # remove any unavailable models or unnecessary series
-    known_matches = {ser: mod for ser, mod in known_matches.items() if ser in org_list}
-    k = {ser: mod for ser, mod in known_matches.items() if mod in available_models}
-    # check if any series are missing from model list
-    if not k:
-        raise ValueError("Horizontal template has no models matching this data!")
-    if len(set(org_list) - set(list(k.keys()))) > 0:
-        all_series = horizontal_classifier(df_train, k)
-    else:
-        all_series = known_matches
 
     forecast_df, u_forecast_df, l_forecast_df = (
         pd.DataFrame(),
@@ -335,6 +376,7 @@ def EnsembleForecast(
     forecasts_runtime,
     prediction_interval,
     df_train=None,
+    prematched_series: dict = None,
 ):
     """Return PredictionObject for given ensemble method."""
     s3list = ['best3', 'best3horizontal', 'bestn']
@@ -373,6 +415,7 @@ def EnsembleForecast(
             forecasts_runtime,
             prediction_interval,
             df_train=df_train,
+            prematched_series=prematched_series,
         )
         return ens_forecast
 
