@@ -161,7 +161,12 @@ def retrieve_regressor(
         return regr
     elif regression_model['model'] == 'HistGradientBoost':
         from sklearn.multioutput import MultiOutputRegressor
-        from sklearn.neighbors import HistGradientBoostingRegressor
+
+        try:
+            from sklearn.experimental import enable_hist_gradient_boosting  # noqa
+        except Exception:
+            pass
+        from sklearn.ensemble import HistGradientBoostingRegressor
 
         regr = MultiOutputRegressor(
             HistGradientBoostingRegressor(
